<?php

namespace Teknisa\FIN\Modules\ContaReceber\Factory;

use Financeiro\Entities\Boletreceber;
use Financeiro\Entities\Contareceber;
use Indeterminado\Entities\Devolucaorec;
use Financeiro\Entities\Adiantareceb;
use Fiscal\Entities\Notafiscal;
use Financeiro\Logic\AtualizaFluxodeCaixa\FluxodeCaixa;
use Financeiro\Entities\Ajbolrec;
use Teknisa\Libs\Util\DateUtil;

class AjusteBoletoFactory {

    // Insere Ajuste do Boleto
    public function inserirAjusteBoleto($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec, $vrajbolrec, $idorigajbrec = '1'){
        /** @var Ajbolrec $ajuste */
        $ajuste = new Ajbolrec();
        $ajuste->setNrorg($nrorg);
        $ajuste->setCdfilial($cdfilial);
        $ajuste->setNrlanbolrec($nrlanbolrec);
        $ajuste->setCdajuctrec($cdajuctrec);
        $ajuste->setVrajbolrec($vrajbolrec);
        $ajuste->setIdorigajbrec($idorigajbrec);
        $ajuste->setDtinclusao(DateUtil::getDataAtual());
        return $ajuste;
    }

    // Busca Ajuste Boleto
    public function getAjuste($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec){
        /** @var Ajbolrec $ajuste */
        $ajuste = Ajbolrec::findOneByNrorgAndCdfilialAndNrlanbolrecAndCdajuctrec($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec);
        return $ajuste;
    }

    // Deleta Ajuste Boleto
    public function deletarAjuste($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec){
        $ajuste = $this->getAjuste($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec);
        $ajuste->delete();
    }

    // Busca Boleto
    public function getBoleto($nrorg, $cdfilial, $nrlanbolrec){
        /** @var Boletreceber $boleto */
        $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg($nrlanbolrec, $cdfilial, $nrorg);
        return $boleto;
    }

    // Atualiza Valor do Boleto
    public function atualizaValorBoleto($nrorg, $cdfilial, $nrlanbolrec, $vrajuste){
        $boleto = $this->getBoleto($nrorg, $cdfilial, $nrlanbolrec);
        
        $boleto->setVrfinalbrec($boleto->getVrfinalbrec() + $vrajuste);
        $boleto->setIdstatbrec('A');
        $boleto->setDslinhabarrec(null);
        $boleto->setCdbarrarec(null);
        $boleto->setIdbolrecalter(1);
        $boleto->setCdnosnumrec(null);
        return $boleto;
    }

    // Atualiza Fluxo de Caixa
    public function atualizaFluxoCaixa($nrorg, $cdfilial, $nrlanbolrec, $vrajuste){
        $contas = Contareceber::findByNrorgAndCdfilibolrecAndNrlanbolrec($nrorg, $cdfilial, $nrlanbolrec);
        /** @var Contareceber $conta */
        foreach($contas as $conta)
            FluxodeCaixa::atualizaFluxo($conta->getCdempresa(), $conta->getCdfimovflrec(), $conta->getCdclassfina(), $conta->getDtmovfluxo(), $vrajuste, 'E', 'Entrada', $nrorg);
    }

    /**
     * Atualiza Ajuste Boleto Receber
     * @param Ajbolrec $ajbolrec
     * @param $vrajbolrec
     * @return mixed
     */
    public function atualizaAjbolrec(
        Ajbolrec $ajbolrec, $vrajbolrec
    ){
        $ajbolrec->setVrajbolrec($vrajbolrec);

        return $ajbolrec;
    }

    /**
     * Busca Conta Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @return Contareceber
     */
    public function buscaContareceber($nrorg, $cdfilial, $nrlanbolrec){

        /** @var Contareceber $contareceber */
        $contareceber = Contareceber::findOneByNrorgAndCdfilibolrecAndNrlanbolrec($nrorg, $cdfilial, $nrlanbolrec);

        return $contareceber;
    }

    /**
     * Busca Nota Fiscal
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @return Notafiscal
     */
    public function buscaNotafiscal($nrorg, $cdfilial, $nrlanctonf){
        /** @var Notafiscal $notafiscal */
        $notafiscal = Notafiscal::findOneByNrorgAndCdfilialAndNrlanctonf($nrorg, $cdfilial, $nrlanctonf);

        return $notafiscal;
    }

    /**
     * Busca Devolução Receber
     * @param $nrorg
     * @param $cdfilibolrec
     * @param $nrlanbolrec
     * @param $cdajuctrec
     * @return Devolucaorec
     */
    public function buscaDevolucaorec($nrorg, $cdfilibolrec, $nrlanbolrec, $cdajuctrec){
        /** @var Devolucaorec $devolucaorec */
        $devolucaorec = Devolucaorec::findOneByNrorgAndcdfilibolrecAndNrlanbolrecAndCdajuctrec(
            $nrorg, $cdfilibolrec, $nrlanbolrec, $cdajuctrec
        );

        return $devolucaorec;
    }

    /**
     * Busca Ajuste Boleto Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $cdajuctrec
     * @return Ajbolrec
     */
    public function buscaAjbolrec($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec){
        /** @var Devolucaorec $devolucaorec */
        $ajbolrec = Ajbolrec::findOneByNrorgAndCdfilialAndNrlanbolrecAndCdajuctrec(
            $nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec
        );

        return $ajbolrec;
    }

    /**
     * Busca Adiantamento Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $cdfiliajbolr
     * @param $cdajuctrec
     * @return Ajbolrec
     */
    public function buscaAdiantaReceb($nrorg, $cdfilial, $nrlanbolrec, $cdfiliajbolr, $cdajuctrec){
        /** @var Devolucaorec $devolucaorec */
        $ajbolrec = Adiantareceb::findOneByNrorgAndCdfilialAndNrlanbolrecAndcdfiliajbolrAndCdajuctrec(
            $nrorg, $cdfilial, $nrlanbolrec, $cdfiliajbolr, $cdajuctrec
        );

        return $ajbolrec;
    }

}
